<?php
/**
 * Main file for initializing admin.
 *
 * @package Omnipress\Admin
 */

namespace Omnipress\Admin;

use Omnipress\Transient;

/**
 * Main admin class to initialize our admin functionalities.
 *
 * @since 1.1.0
 */
class Admin {

	/**
	 * Current object instance.
	 *
	 * @var Admin
	 */
	protected static $instance;

	/**
	 * Current object instance.
	 *
	 * @return Admin
	 */
	public static function init() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Class construct.
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'force_cold_boot' ) );
		add_action( 'admin_menu', array( $this, 'register_admin_menus' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_action( 'admin_footer', array( $this, 'add_app_root' ) );
	}

	/**
	 * Handle the "force_cold_boot" request.
	 *
	 * @return void
	 */
	public function force_cold_boot() {

		if ( empty( $_GET['force_cold_boot'] ) ) {
			return;
		}

		if ( empty( $_GET['op_nonce'] ) ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['op_nonce'] ) ), '_omnipress_nonce' ) ) {
			return;
		}

		do_action( 'omnipress_doing_force_cold_boot' );

		$transient = new Transient();
		$transient->delete_all();

		wp_safe_redirect( remove_query_arg( array( 'force_cold_boot', 'op_nonce' ) ) );
		exit;
	}

	/**
	 * Adds the required html element for library app modal in post/page editor.
	 *
	 * @return void
	 */
	public function add_app_root() {
		?>
		<div id="omnipress"></div>
		<?php
	}

	/**
	 * Register our admin menus.
	 *
	 * @return void
	 */
	public function register_admin_menus() {
		add_menu_page(
			__( 'Omnipress', 'omnipress' ),
			__( 'Omnipress', 'omnipress' ),
			'manage_options',
			'omnipress',
			function() {},
			'data:image/svg+xml;base64,' . base64_encode(  @file_get_contents( OMNIPRESS_PATH . 'assets/images/omnipress-dashboard-menu-icon.svg' ) )
		);
	}

	/**
	 * Enqueue admin scripts and styles.
	 *
	 * @return void
	 */
	public function enqueue_scripts( $hook_prefix ) {

		if (
			'post-new.php' !== $hook_prefix &&
			'post.php' !== $hook_prefix &&
			'site-editor.php' !== $hook_prefix &&
			'toplevel_page_omnipress' !== $hook_prefix
		) {
			return;
		}

		do_action( 'omnipress_before_admin_scripts', $hook_prefix );

		$assets = include OMNIPRESS_PATH . 'assets/admin/index.asset.php';

		wp_enqueue_style( 'omnipress-admin-style', OMNIPRESS_URL . 'assets/admin/admin.css', array( 'wp-components' ), $assets['version'], 'all' );

		wp_enqueue_script( 'omnipress-admin-script', OMNIPRESS_URL . 'assets/admin/index.js', $assets['dependencies'], $assets['version'], true );

		$current_user = get_userdata( get_current_user_id() );

		$localize = array(
			'nonce'            => wp_create_nonce( '_omnipress_nonce' ),
			'omnipressVersion' => OMNIPRESS_VERSION,
			'isOmnipressPage'  => 'toplevel_page_omnipress' === $hook_prefix,
			'urls'             => array(
				'home'        => home_url(),
				'wpDashboard' => admin_url(),
			)
		);

		if ( $current_user ) {
			$localize['user'] = array(
				'firstName' => $current_user ? $current_user->first_name : '',
				'username'  => $current_user ? $current_user->user_login : '',
				'avatarURL' => $current_user ? get_avatar_url( $current_user->ID ) : ''
			);
		}

		wp_localize_script( 'omnipress-admin-script', '_omnipress', apply_filters( 'omnipress_localize_admin_script', $localize ) );

		do_action( 'omnipress_after_admin_scripts', $hook_prefix );
	}

}
